#!/usr/bin/python3

import os
import gettext

def _strip_split_and_recombine(comma_separated):
    return ";".join([word.strip() for word in comma_separated.split(",")])

def generate(domain, path, filename, prefix, name, comment, suffix, genericName=None,
             keywords=None, append=False):
    """
    Create a localized .desktop file.

    params:

    `domain` - translation domain

    `path` - path to .mo files

    `filename` - .desktop file to create

    `prefix` - .desktop file content to write first, won't be localized

    `name` - Name key in the .desktop file (localized)

    `comment` - Comment key in the .desktop file (localized)

    `suffix` - .desktop file content to write last, won't be localized

    `genericName` - GenericName key in the .desktop file (localized)

    `keywords` - Keywords key in the .desktop file (localized)

    `append` - bool deciding whether `filename` will be overwritten or appended to

    """
    os.environ['LANGUAGE'] = "en_US.UTF-8"
    gettext.install(domain, path)
    if append:
        desktopFile = open(filename, "a")
    else:
        desktopFile = open(filename, "w")

    desktopFile.writelines(prefix)

    desktopFile.writelines("Name=%s\n" % name)
    for directory in sorted(os.listdir(path)):
        mo_file = os.path.join(path, directory, "LC_MESSAGES", "%s.mo" % domain)
        if os.path.exists(mo_file):
            try:
                language = gettext.translation(domain, path, languages=[directory])
                language.install()
                if (_(name) != name):
                    desktopFile.writelines("Name[%s]=%s\n" % (directory, _(name)))
            except:
                pass

    if comment is not None:
        desktopFile.writelines("Comment=%s\n" % comment)
        for directory in sorted(os.listdir(path)):
            mo_file = os.path.join(path, directory, "LC_MESSAGES", "%s.mo" % domain)
            if os.path.exists(mo_file):
                try:
                    language = gettext.translation(domain, path, languages=[directory])
                    language.install()
                    if (_(comment) != comment):
                        desktopFile.writelines("Comment[%s]=%s\n" % (directory, _(comment)))
                except:
                    pass

    if keywords is not None:
        formatted = _strip_split_and_recombine(keywords)
        desktopFile.writelines("Keywords=%s\n" % formatted)
        for directory in sorted(os.listdir(path)):
            mo_file = os.path.join(path, directory, "LC_MESSAGES", "%s.mo" % domain)
            if os.path.exists(mo_file):
                try:
                    language = gettext.translation(domain, path, languages=[directory])
                    language.install()
                    if (_(keywords) != keywords):
                        translated = _strip_split_and_recombine(_(keywords))
                        desktopFile.writelines("Keywords[%s]=%s\n" % (directory, translated))
                except:
                    pass

    if genericName is not None:
        desktopFile.writelines("GenericName=%s\n" % genericName)
        for directory in sorted(os.listdir(path)):
            mo_file = os.path.join(path, directory, "LC_MESSAGES", "%s.mo" % domain)
            if os.path.exists(mo_file):
                try:
                    language = gettext.translation(domain, path, languages=[directory])
                    language.install()
                    if (_(genericName) != genericName):
                        desktopFile.writelines("GenericName[%s]=%s\n" % (directory, _(genericName)))
                except:
                    pass

    desktopFile.writelines(suffix)
    os.environ['LANGUAGE'] = "en_US.UTF-8"
    gettext.install(domain, path)

def generate_polkit_policy(domain, path, filename, prefix, message, suffix, append=False):
    """
    Create a localized polkit action .policy file.

    params:

    `domain` - translation domain

    `path` - path to .mo files

    `filename` - .policy file to create

    `prefix` - .policy file content to write first, won't be localized

    `message` - message element in the .policy file (localized)

    `suffix` - .policy file content to write last, won't be localized

    `append` - bool deciding whether `filename` will be overwritten or appended to

    """

    os.environ['LANGUAGE'] = "en_US.UTF-8"
    gettext.install(domain, path)
    if append:
        desktopFile = open(filename, "a")
    else:
        desktopFile = open(filename, "w")

    desktopFile.writelines(prefix)

    desktopFile.writelines("    <message>%s</message>\n" % message)
    for directory in sorted(os.listdir(path)):
        mo_file = os.path.join(path, directory, "LC_MESSAGES", "%s.mo" % domain)
        if os.path.exists(mo_file):
            try:
                language = gettext.translation(domain, path, languages=[directory])
                language.install()
                if (_(message) != message):
                    desktopFile.writelines("    <message xml:lang=\"%s\">%s</message>\n" % (directory, _(message)))
            except:
                pass

    desktopFile.writelines(suffix)
    os.environ['LANGUAGE'] = "en_US.UTF-8"
    gettext.install(domain, path)