#!/usr/bin/python3

"""
Wrapper for non-interactive Synaptic that handles elevation via pkexec while
preserving the user's environment

Functions:
  * update_cache
  * install
"""

import os
import subprocess
import sys

from mintcommon import proxysettings

def _conf_workaround(enable):
    """ Workaround for Synaptic ignoring command line arguments in favour of its configuration file """
    SYNAPTIC_CONF = "/root/.synaptic/synaptic.conf"
    WORKAROUND = "/root/.synaptic/synaptic-workaround.conf"
    try:
        if enable and os.path.isfile(SYNAPTIC_CONF):
            os.rename(SYNAPTIC_CONF, WORKAROUND)
        elif not enable and not os.path.isfile(SYNAPTIC_CONF) and \
            os.path.isfile(WORKAROUND):
            os.rename(WORKAROUND, SYNAPTIC_CONF)
    except:
        pass

def update_cache(xid):
    if not os.getuid() == 0:
        return _elevate("update", xid)
    _conf_workaround(True)
    cmd = ["/usr/sbin/synaptic", "--hide-main-window", "--update-at-startup", "--non-interactive"]
    if xid:
        cmd.extend(["--parent-window-id", str(xid)])
    try:
        p = subprocess.run(cmd)
        result = p.returncode
    except:
        result = -1
    _conf_workaround(False)
    return result

def install(xid, selections_file, closeZvt=True):
    if not os.getuid() == 0:
        return _elevate("install", xid, selections_file, closeZvt)
    _conf_workaround(True)
    cmd = ["/usr/sbin/synaptic", "--hide-main-window", "--non-interactive",
           "-o", f"Synaptic::closeZvt={'true' if closeZvt else 'false'}",
           "--set-selections-file", selections_file]
    if xid:
        cmd.extend(["--parent-window-id", str(xid)])
    try:
        p = subprocess.run(cmd, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
        result = p.returncode
    except:
        result = -1
    _conf_workaround(False)
    return result

def _elevate(mode, xid="", selections_file="", closeZvt=True):
    cmd = ["/usr/bin/pkexec", "/usr/lib/python3/dist-packages/mintcommon/synaptic.py", mode, str(xid), selections_file, "closeZvt" * closeZvt]
    proxysettings.set_environment_variables()
    for env in ("HOME", "DISPLAY", "XAUTHORITY", "http_proxy", "https_proxy", "ftp_proxy", "all_proxy", "no_proxy"):
        if env in os.environ:
            cmd.append(f"{env}={os.environ[env]}")
    p = subprocess.run(cmd)
    return p.returncode

def _set_environment():
    for arg in sys.argv[1:]:
        arg = arg.split("=", 1)
        if arg[0] in ("HOME", "DISPLAY", "XAUTHORITY", "http_proxy", "https_proxy", "ftp_proxy", "all_proxy", "no_proxy"):
            os.environ[arg[0]] = arg[1]

_set_environment()
if __name__ == "__main__":
    if not os.getuid() == 0 or len(sys.argv) < 2:
        sys.exit(1)
    try:
        if sys.argv[1] == "update":
            update_cache(sys.argv[2])
        elif sys.argv[1] == "install":
            install(sys.argv[2], sys.argv[3], sys.argv[4] == "closeZvt")
    except:
        pass
